function parameters_limcom(chosen_model,kappa)

%parameters_limcom(chosen_model) declares the calibration parameters for
%the limited commitment model with capital.
%
%Input:
%   chosen_model        Type of model chosen
%   kappa               Information parameter
%Output:    None
%
%Author: Swapnil Singh and Christian Stoltenberg
%
%Version: 
%February 1, 2019
%

p.R =1.025;%;  %Targeted interest rate leading to capital-income of 2.5
p.sigma=1;                              %Parameter specification of utility function
p.rho = 0.9989;                         %Persistence parameter
p.delta = 0.1357 ;%capital depreciation rate
p.alpha = 0.3;                          %capital share
p.AA = 1.0644;      %technology parameter
p.np = 200;                             %number of grid points for promised values
p.na = p.np;                             %Grid points for savings in autarky
p.npers =3;                            %Number of persistent states
p.ntrans = 2;                           %Number of transitory states
p.S = p.npers * p.ntrans;               %Total number of income states
p.pcrit=1e-10;                          %Invariant distribution, minimum probability of state to be kept
p.crit = 1e-6;                            %Convergence criterion
p.tauchen = 0;                           %Which method to use for discretizationm of income
p.cmin = 1e-5;                          %minimum consumption level
p.back = 0;                                %Return to insurance
p.screw=0;                               %Expropriation in aut
p.r_aut=(p.R-1);                         %Exogenous interest in outside option


if strcmp(chosen_model, 'pers_limcom_wi_capital_outside_test_fgrid') &&  nargin == 2
    p.kappa=kappa;                                  %Uninformative is 1/p.S
    p.Sinf = p.S^2;                                 %Number of income states are squared in presence of information
end




%save the parameters
cd 'Limcom'
filename = strcat('parameters_', chosen_model);
save(filename);
cd ..

end