function pers_limcom_wi_capital_outside_test_fgrid()



%1. Load the parameters
%----------------------
load('parameters_pers_limcom_wi_capital_outside_test_fgrid.mat');     
strintr = num2str(p.R);
strnpers = num2str(p.npers);
strntrans = num2str(p.ntrans);
strkappa = num2str(p.kappa);
ff=mfilename;
savefilename = strcat(ff,'_', strntrans, 'bynoreturn', strnpers,'_', strintr, '_kappa_', strkappa);         %In this file name all results will be saved
savefilename(savefilename == '.') = [];

fprintf('Number of persistent income states = %i\n', p.npers);
fprintf('Number of transitory income states = %i\n', p.ntrans);
fprintf('Persistent parameter = %f\n', p.rho );
fprintf('Interest rate = %f\n', p.R);
fprintf('Number of grid points = %i\n', p.np);
fprintf('Results are saved in filename = %s \n', savefilename);

%2. Income Process
%-----------------
[Z,Q, SSdist] = zprocess(p);

%for further analysis I need Z in a row form

if size(Z,1) > size(Z,2)
   Z = Z';      %Z is a row vector now 
end

if size(SSdist, 1)< size(SSdist,2)
   SSdist = SSdist';            %SSdist is a column vector now 
end
Zbar = SSdist'* Z';

pinf = zeros(p.S^2);

    for i1 = 1 : p.S^2              %today state
       for i2 = 1 : p.S^2           %tomorrow state
           income_col = ceil(i2/p.S);
           state_info_col = i2 - floor((i2-1e-6)/p.S)*p.S;
           income_row = ceil(i1/p.S);
           state_info_row = i1 - floor((i1-1e-6)/p.S)*p.S;

           ind1 = isequal(Z(income_col), Z(state_info_row));
           ind2 = 1 - ind1;
           pitemp = Q(income_row,:);
           pinf(i1,i2) = (p.kappa^ind1 * Q(income_row, income_col) * ((1 - p.kappa)/(p.S-1))^ind2/(p.kappa * pitemp(state_info_row) + (1-p.kappa)/(p.S-1)* (1 - pitemp(state_info_row))));
       end
    end
    
   pinf = pinf.*repmat(Q,p.S,p.S); %WARNING: this works only if the income transitionmatrix is symmetric (as in our calibration). If not, one needs to compute the consistent signal transition here (see Section A.3 for details).
    if max(sum(pinf,2) - 1) > 1e-10 
       error('The columns of transition matrix are not adding upto 1'); 
    end
    
    
  

   
    
    
    
    
    %manipulate the variables to take into account the information
   Q = pinf;
    Z = kron(Z, ones(p.S,1));
    
    p.S = p.Sinf;

%3. Apply the Ridder algorithm
%-----------------------------
    
    %3.1 Choose two extreme points of beta
    %-------------------------------------
  b1 =0.8930; b2 = 0.95;
    
    tic;
    d1 = solve_model(b1,p,Z,Q,Zbar,SSdist,0,savefilename);
    
 
    disp('----------------------------------------------');
    fprintf('Time taken to complete b1: %f\n', toc);
  stop
    tic;
    disp('----------------------------------------------');
    d2 = solve_model(b2,p,Z,Q,Zbar,SSdist,0,savefilename);
    
    
   
    
    fprintf('Time taken to complete b2: %f\n', toc)
    disp('----------------------------------------------');
    
    %check whether root lies in between b1 and b2
    if d1*d2 > 0
        error('Root is not in between the given values of discount rates');
    end
    
    
    if d1 == 0 || d2 == 0
       error('You are already at the solution point');
    end
    
    bl = b1; bh = b2;
    dl = d1; dh = d2;
    conv = 1e-6;
    iter = 0;
    
    tic;
    disp('----------------------------------------------');
    while true
       bm = 0.5*(bl + bh);
       dm = solve_model(bm,p,Z,Q,Zbar,SSdist,0,savefilename);
       if sign(dm^2 - dl*dh) > 0
          sq = sqrt(dm^2 - dl*dh);
       else
           error('There is something wrong. Please check');
       end
       if sq == 0
          error('Dinominator is zero'); 
       end
       
       bnew = bm + (bm - bl)*sign(dl-dh)*dm/sq;
       if abs(bnew - conv) < 1e-4
         fprintf('Converged subjective discount factor = %f\n', bnew)
         dnew = solve_model(bnew,p,Z,Q,Zbar,SSdist,1,savefilename);
         p.beta = bnew;
         cd 'Results'
         save(savefilename,'p', '-append');
         cd ..
         
         break;
       end
       fprintf('The difference between old and new beta is = %f\n',(abs(bnew - conv)));
       conv = bnew;
       dnew = solve_model(conv,p,Z,Q,Zbar,SSdist,0,savefilename);
       
       if abs(dm)*sign(dnew) ~= dm
          bl = bm;
          dl = dm;
          bh = conv;
          dh = dnew;
       elseif abs(dl)*sign(dnew) ~= dl
           bh = conv;
           dh = dnew;
       elseif abs(dh)*sign(dnew) ~= dh
           bl = conv;
           dl = dnew;
       else
           error('There is something wrong with the code')
       end
       iter = iter + 1;
    end
    
    fprintf('Time taken to complete the  Ridder: %f\n', toc);
    disp('---------------------------------------------------');



end


%----------------------------------------------------------------%
%--------- Solve Model Code -------------------------------------%
%----------------------------------------------------------------%
function ed = solve_model(b,p,Z,Q,Zbar,SSdist,i,savefilename)
%global vf_aut
%ed = solve_model(R,p,Z,Q,Zbar,i,savefilename) is the function which solves
%the Excess Demand of capital for the given R
%
%Inputs: 
%           R                   Gross interest rate
%           p                   Structure of parameters
%           Z                   Income states
%           Q                   Income transition matrix
%           Zbar                Average income
%           SSdist              Stationary Distribution of Income Shock
%           i                   Indicator to save the results
%           savefilename        Filename in which all the results are saved
%
%Output:
%           ed                  Excess demand
%


%1. Function handles for utility function
%----------------------------------------
if p.sigma==1
    u = @(c) log(c);                    %logarithmic utility
    up1 = @(c) 1./c;                    %first derivative of the  utility function
    u1 = @(u) exp(u);                   %inverse of the logarithmic utility
    u1p = @(u) exp(u);                  %derivative of the inverse of the logarithmic utility
else
    u= @(c) (c.^(1-p.sigma)-1)/(1-p.sigma);                 % CES utility
    up1 = @(c) c.^(-p.sigma);                               %first derivative of the utility function
    u1= @(u) ((1-p.sigma)*u+1).^(1/(1-p.sigma));            % inverse function of the utility function
    u1p = @(u) ((1-p.sigma)*u+1).^(p.sigma/(1-p.sigma));    %derivative of the inverse of the logarithmic utility
end
disp('-----------------------------------------');
fprintf('Running for beta = %f\n',b);
disp('-----------------------------------------');
p.beta = b;
R = p.R;
r = R-1;    %Net Interest rate
k_agg = ((r + p.delta)/(p.AA*p.alpha))^(1/(p.alpha - 1));       %aggregate demand of capital
w = (1 - p.alpha) *p.AA* k_agg^p.alpha;                      %wage rate
q = 1/R;
parrow = (1/R)* Q;              %price of arrow securities


%transform the income
Z = w * Z(:)';
Zbar = w *  Zbar;        %the mean of the income will also change




 %computation of the outside option where agents are allowed to come
    %back and save. The probability of coming back is p.back
    %2.1 Value function iteration to compute the value of the outside option
    %-----------------------------------------------------------------------
    %Algorithm Parameters(stored in structure a)
    %----------------------------------------------
    a.nm = p.na;                         %number of grid points for savings
    a.mmin = 0;                        %lowest value on the savings grid
    a.mmax = 30;                         %highest value on the savings grid
    critaut=1;

    %constant stream of consumption in case of reverting back
    c_bar_frs=p.AA*k_agg^(p.alpha)-p.delta*k_agg;
    
    %Grid for Savings
    %-------------------
    SI = sort(unique([linspace(a.mmin,a.mmax,a.nm)';0]));    %savings grid
    index_SI = find(SI==0);
    a.nm = length(SI);
    
    
    %compute the value function when the individual comes back to insurance
    %----------------------------------------------------------------------
    v_ret = zeros(a.nm,1);
    
     %Operational matrices
    %-----------------------
    o = ones(a.nm,1);
    oo = ones(a.nm);
    H = cell(p.S,1);

    S1 = repmat(SI, 1, a.nm);

    S2 = repmat(SI', a.nm,1);
    
    A_ret = c_bar_frs + (1 + r)* S1 - S2;
    A_ret(A_ret <= 0) = NaN;
    A_ret = (1-p.beta)*u(A_ret);
    while true
       temp = A_ret + p.beta * repmat(v_ret', a.nm,1);
       [temp_v,~] = max(temp,[],2);
       diff = max(abs(temp_v(:) - v_ret(:)));
       if diff < 1e-4
          break; 
       end
       v_ret = temp_v;
    end
    v_ret = repmat(v_ret(:), 1,p.S)';
    clearvars temp_v temp_d
    
    
    
    
    
    
    
    
    %Storage matrices
    %-------------------
    v = zeros(a.nm, p.S) ;         %value function



    %Operational matrices
    %-----------------------
    o = ones(a.nm,1);
    oo = ones(a.nm);
    H = cell(p.S,1);

    S1 = repmat(SI, 1, a.nm);

    S2 = repmat(SI', a.nm,1);
    A = [];
    for j = 1 : p.S
       H{j} =(1-p.screw)*(1+r)*S1 +  oo*Z(j) - S2;
       H{j}(H{j}<=0) = NaN;
       H{j} = (1-p.beta)*u(H{j});     
       A = [A;H{j}];
    end
   
    
    while critaut >= p.crit
       tempaut = (A + p.beta*(1-p.back)* kron(Q, o) * v'+p.beta*p.back * kron(Q, o) * v_ret );
       [tv,d] = max(tempaut, [], 2);
       tv = reshape(tv,a.nm,p.S );
       d = reshape(d,a.nm,p.S);
       critaut = max(max(abs(tv - v)));
       v = tv;
    end

    %2.2 Autarky value
    %-----------------
    waut=v(index_SI,:)';
    
    
    
%3. Solve the model
%------------------
options_fsolve = optimoptions(@fsolve, 'Display', 'off');       %options for fsolve

asset = repmat(ones(p.na,p.S), [1 1 p.S]);                      %storage matrix for the asset
vp0=repmat(linspace((1-p.beta)/Z(1),(1-p.beta)*Z(end),p.na)', 1,p.S);                 %initial guess for the derivative of  the value function
cons = 0.1 * ones(p.na,p.S);                                    %initial guess for the consumption
a_lim =zeros(p.S,1);

%vf = zeros(p.na,p.S);                                           %initial guess for the value function

agrid = linspace(-1,40,p.na)';  %asset grid baseline, 200 gridpoints  


[ma,mZ] = ndgrid(agrid, Z(:)');                                 %meshgrid for assets and income
    [ma3,mZp3,mZf3] = ndgrid(agrid, Z(:)', Z(:)');
    mZ = mZ + repmat(linspace(0,1e-3,p.S), p.na,1);
    mZp3 = mZp3 + repmat(repmat(linspace(0,1e-3,p.S),p.na,1), [1 1 p.S]);% + repmat(reshape(linspace(0,1e-3,p.S), [1 1 p.S]), p.na,p.S);
    mZf3 = mZf3  + repmat(reshape(linspace(0,1e-3,p.S), [1 1 p.S]), p.na,p.S);
  

%convergence for the borrowing limit
while true    %this is the outermost while loop for  the borrowing limit convergence
     F_vp0 = griddedInterpolant(ma,mZ,vp0); %NOTE: griddedInterpolant uses extrapolation as default!
   vp0=F_vp0(ma, mZ);
    F_a =griddedInterpolant(ma3,mZp3,mZf3,asset); %NOTE: griddedInterpolant uses extrapolation as default!
    asset=F_a(ma3,mZp3,mZf3);
    F_c = griddedInterpolant(ma,mZ,cons);%NOTE: griddedInterpolant uses extrapolation as default!
    cons=F_c(ma,mZ);
    
    cons(cons < p.cmin) = p.cmin;
    income_states=p.S;
    asset_states=p.na;
    
  %while true    %this is the while loop for the value function convergence  
        
       while true   %this is the while loop for the value function derivative convergence

        temp_asset = asset;     %auxiliary matrix for asset
        temp_cons = cons;       %auxiliary matrix for consumption
        F = griddedInterpolant(ma, mZ, vp0);%NOTE: griddedInterpolant uses extrapolation as default!

                while true   %this is the while loop for the asset and consumption convergence
                    
                    parfor ia = 1 : asset_states    %for loop for the assets

                        for iz = 1 : income_states     %for loop for the income

                            inia = agrid(ia);           %initial asset
                            iniy = Z(iz);               %initial income
                            prices = parrow(iz,:);      %arrow prices from the present income state
                            tasset = reshape(asset(ia,iz,:), p.S,1);
                            index = tasset <= a_lim;
                            tasset(index) = [];
                            x0 =  [u(cons(ia,iz));tasset];
                            [sol,~,~] = fsolve(@(x)fsolve_equation(x, inia, iniy, F,...
                                    p, prices, w, Z, q, index, a_lim), x0, options_fsolve);
                            temp_cons(ia,iz) = u1(sol(1));

                            sol(1) = [];

                            temp_sol = zeros(p.S,1);  
                            temp_sol(index) = a_lim(index);
                            temp_sol(~index) = sol;
                            index_2 = temp_sol <= a_lim;
                            temp_sol(index_2) = a_lim(index_2);
                            temp_asset(ia,iz,:) = reshape(temp_sol,[1 1 p.S]);
                            
                        end
                    end

                    criteria_initial = max([max(abs(cons(:) - temp_cons(:))) ...
                        max(abs(asset(:) - temp_asset(:)))  ]);
                    
                     %fprintf('Criteria for convergence in asset and consumption is = %f\n', criteria_initial);
                    if criteria_initial < 1e-4
                       break; 
                    end


                       adj_policy = 1;
                       cons = adj_policy * temp_cons + (1 - adj_policy) * cons;
                       asset = adj_policy * temp_asset + (1 - adj_policy) * asset; 
%                        

                end   % end of the inner while loop

        
        vp1 = (1 - p.beta) * up1(cons);

        criteria_vp =  max(abs(vp1(:) - vp0(:)));
        criteria_mid=max(criteria_vp);
        fprintf('Criteria for convergence in derivative of value function is: %f\n', criteria_mid);
        if criteria_mid < 1e-4
           break; 
        end


        vp0 = vp1;
       end
    
   
        %compute the transition matrix
        alyp = zeros(p.na, p.S, p.S);
        ahyp = alyp;

        for i1 = 1 : p.na    %present asset

            for i2 = 1 : p.S     %present income


                for i3 = 1 : p.S    %future income

                    temp = find(agrid <= temp_asset(i1,i2,i3));

                    if isempty(temp)
                       alyp(i1,i2,i3) = min(agrid);
                    else
                        alyp(i1,i2,i3) = max(agrid(temp));
                    end

                    temp = find(agrid > temp_asset(i1,i2,i3));

                    if isempty(temp)
                       ahyp(i1,i2,i3) = max(agrid);
                    else
                        ahyp(i1,i2,i3) = min(agrid(temp));
                    end
                end
            end
        end

        alphayp =  (ahyp-asset)./(ahyp - alyp);
        alphayp(alphayp == Inf) = 1;
        alphayp(alphayp == -Inf) = 1;
        trans_matrix = zeros(p.na*p.S);    %transition matrix

        for ir = 1 : p.na*p.S
           for ic = 1 : p.na*p.S
              nyp = ceil(ic/p.na);
              ny = ceil(ir/p.na);
              nwp= ic - floor((ic-1e-6)/p.na)*p.na;
              nw = ir - floor((ir-11e-6)/p.na)*p.na;

              assetp = agrid(nwp);


              if assetp == alyp(nw,ny,nyp) 
                 trans_matrix(ir,ic) = Q(ny,nyp)*alphayp(nw,ny,nyp);
              elseif assetp  == ahyp(nw,ny,nyp)
                  trans_matrix(ir,ic) = Q(ny,nyp)*(1-alphayp(nw,ny,nyp));
              else
                  trans_matrix(ir,ic) = 0;
              end


           end

        end
        



        %calculate the value function
%         vf = reshape((eye(p.na*p.S) - p.beta * trans_matrix)\((1-p.beta)*u(cons(:))), p.na,p.S);
        tvf = reshape((eye(p.na*p.S) - p.beta * trans_matrix)\((1-p.beta)*u(cons(:))), p.na,p.S);
%         
        %criteria_vf = max(abs(vf(:) - tvf(:)));
%         
        vf = tvf;
    
        
%         fprintf('THE CONVERGENCE CRITERIA FOR VALUE FUNCTION IS: %f\n', criteria_vf);
%         
%         if criteria_vf < 1e-4
%            break; 
%         end

  %end
    
    
    %gridded Interpolant of the value function
    F_vf = griddedInterpolant(ma, mZ, vf);%NOTE: griddedInterpolant uses extrapolation as default!
    
   
   
    

    
    
    
%     if waut(end)>u(Zbar)
%         break
%     end
    
    
    %waut=v(15,:)';
   
    %gridded Interpolant of the value function
    %F_vf = griddedInterpolant(ma, mZ, vf);
    %compute the new borrowing constraints
    new_a_lim = zeros(p.S,1);

    for i1 = 1 : p.S
       constraint = @(x) F_vf(x, Z(i1)) - waut(i1);
       [new_a_lim(i1),~, exitflag] = fsolve(@(x)constraint(x), a_lim(i1), options_fsolve);
       if exitflag ~= 1
          new_a_lim(i1) = a_lim(i1); 
       end
    end
    
    criteria_bl = max(abs(a_lim-new_a_lim));
    
    fprintf('CONVERGENCE IN BORROWING LIMIT IS: %f\n', criteria_bl);
    a_lim = 1*new_a_lim+0*a_lim;
    disp(max(a_lim));
    disp(min(a_lim));
    
    
    if criteria_bl < 6e-4
       break; 
    end
    
end


    


%compute the stationary distribution based on the transition matrix
%computed above
[pp,eval] = eigs(trans_matrix',1);
pp = pp / sum(pp);
pp(pp < p.pcrit) = 0;
p_st = pp(pp>0);
lcons=log(cons);
IR=1- (pp(:)'*lcons(:).^2 - (pp(:)'*lcons(:))^2)/0.365475

%compute the aggregate supply of capital
temp_Q = repmat(reshape(Q, [1 p.S p.S]),[p.na,1,1]);
temp_1 = sum(temp_Q.*asset,3);
temp_1=temp_1(:);
% as = (temp_1(:)'* pp(:));
% ed = (as*q-k_agg)/k_agg;
%ed = as;
%ed2 = abs(ed);
ed=(cons(:)'*pp+p.delta*k_agg-p.AA*k_agg^p.alpha)/k_agg;


disp('--------------------------------');
fprintf('Excess demand is: %f\n', ed);
disp('--------------------------------');


cd 'Results'
    save(savefilename);
   cd ..
if abs(ed) < 5e-4
   disp('Code is finished')
   stop
end


end



function f = fsolve_equation(x,inia, iniy, F,  p, prices, w,Z,q, index,a_lim)
%1. Function handles for utility function
%----------------------------------------
if p.sigma==1
    u = @(c) log(c);                    %logarithmic utility
    up1 = @(c) 1./c;                    %first derivative of the  utility function
    u1 = @(u) exp(u);                   %inverse of the logarithmic utility
    u1p = @(u) exp(u);                  %derivative of the inverse of the logarithmic utility
else
    u= @(c) (c.^(1-p.sigma)-1)/(1-p.sigma);                 % CES utility
    up1 = @(c) c.^(-p.sigma);                               %first derivative of the utility function
    u1= @(u) ((1-p.sigma)*u+1).^(1/(1-p.sigma));            % inverse function of the utility function
    u1p = @(u) ((1-p.sigma)*u+1).^(p.sigma/(1-p.sigma));    %derivative of the inverse of the logarithmic utility
end


    if length(x) == 1
       f = w * iniy + inia - u1(x(1))   - prices * a_lim;
    elseif length(x) > 1
        prices_not_bound = prices(~index);
        prices_bound = prices(index);

        c = p.beta * F(x(2:end), Z(~index)') - q * (1 - p.beta)  * up1(u1(x(1)));
        ceq = w * iniy + inia - u1(x(1)) - prices_not_bound * x(2:end) - prices_bound * a_lim(index);
        f = [c;ceq];
    end
end