function simulation_limcom()




%0. Preliminaries
%----------------                    
clc; clear all; close all;
addpath('Include', 'Data', 'Results' , 'Limcom');
warning('off')

%fix the seed of random number generation
s = RandStream('mcg16807','Seed',0);
RandStream.setGlobalStream(s);


TT = 300000;                    %length of the simulation
nburn = 100000;                 %number of simulation burned


fprintf('Total number of simulation period = %i\n', TT);
fprintf('Number of initial simulations burned = %i\n', nburn);

%file name for the no information case

fn_limcom_ni='pers_limcom_ni_capital_outside_test_2by7_1025200gp_morestates';

%filename for the information case
vkappa = [0.4375];                    %add the information case here
nkappa = length(vkappa);            
fn_limcom_wi = cell(nkappa,1);      %storage cells for the with information filenames
load('parameters_pers_limcom_wi_capital_outside_test_fgrid','p');

for i1 = 1 : nkappa
    strintr = num2str(p.R);
    strnpers = num2str(p.npers);
    strntrans = num2str(p.ntrans);
    strkappa = num2str(vkappa(i1));

     fn_limcom_wi{i1}=  strcat('pers_limcom_wi_capital_outside_test_fgrid_2bynoreturn3_1025_kappa_',num2str(vkappa(i1)) );
    fn_limcom_wi{i1}(fn_limcom_wi{i1} == '.') = [];
end
savefilename_simul = 'final_results_limcom_simul_capital';

%storage matrices
vbetta_simul = zeros(nkappa+1,4);       %storage matrices for the regression coefficient
vtstat_simul = zeros(nkappa+1,4);       %storage matrices for the t-statistic coefficient
vse_simul = zeros(nkappa+1,4);          %storage matrices fot the standard error coefficient
IR_simul = zeros(nkappa+1,1);           %storage matrix for insurance ratios
mcZ_simul = zeros(nkappa+1,p.S);    %storage matrix for the conditional mean of income
scZ_simul = zeros(nkappa+1,p.S);    %storage matrix for the conditional std. dev. of income
vpvalue_simul = zeros(nkappa+1,4);    %storage matrix for the p-value of regression coefficient            
%-----------------------------------------------------------------
%-------No Information Economy------------------------------------
%-----------------------------------------------------------------
load(fn_limcom_ni)
disp('-----------------------------------------------------------');
disp('Limited Commitment No Information model is simulated');

            

            vbetta = zeros(1, 4);
            vse = zeros(1, 4);
            vtstat = vse;

            %storage matrices
            mshock = zeros(1,TT);                   %storage vector for the chain
            mg = zeros(1, TT);                      %storage vector for the promised values
            mcons = zeros(1, TT-1);                 %storage vector for the delivered consumption today

            %simulate the shock process
            [~, state] = markov(Q, TT,1,Z(:)');
            state = [[1;zeros(numel(Z)-1,1)] state];
            for j1 = 1 : TT
               mshock(j1) = find(state(:,j1) == 1); 
            end
            mnT = Z(mshock);

            %initial asset
            mg(1) = agrid(1);

            
            for j2 = 1 : TT-1 
                 
                %interpolate for the consumption
                tempc = cons(:,mshock(j2));
                mcons(j2) = interp1(agrid,tempc,mg(j2), 'linear');
    
                %interpolate for the asset
                tempa = asset(:,mshock(j2), mshock(j2+1));
                mg(j2+1) = interp1(agrid,tempa,mg(j2),'linear');
            end


            %burn the initial simulation
            mcons_new = mcons(nburn+1:end);
            mnT_new = mnT(nburn+1:end-1);
            mshock_new = mshock(nburn+1:end-1);

            %log consumption and income
            lcons = log(mcons_new);
            ly  = log(mnT_new);


           %compute the unconditional variance of consumption and income
           v_vcvZ = var(lcons)/var(ly);
           IR_simul(1) = 1 - var(lcons)/var(ly);

           %Compute regression on future income growth
           lc=log(mcons_new);
           ldc = lc(2:end) - lc(1:end-1);
           ldc=ldc(:);           
           ly  = log(mnT_new);
           yh = lagmatrix(ly(:),[0, -1, -2 -3 -4]);% help matrix income
           gry = yh(:,2:5) - yh(:,1:4);
           indexgry = isnan(gry(:,end));
           gry(indexgry,:)=[];
           ldcf=ldc(1:length(gry));  
           mxf = [ones(length(ldcf),1), gry];%constant, coeff. on future income growth,  current income growth
           bettaf = (mxf'* mxf)\mxf'* ldcf(:);
           v_bettaf_no_info_LC = [bettaf(2), bettaf(3) bettaf(4) bettaf(5)]; %coefficient on future income growth
           sef=(ldcf-mxf*bettaf)'*(ldcf-mxf*bettaf);
           betta_sef=(mxf'*mxf).\(sef*ones(5));
           se_bettaf_coef=diag(betta_sef);
           t_value=v_bettaf_no_info_LC(:)./se_bettaf_coef(2:5);

           vbetta_simul(1, :) = v_bettaf_no_info_LC;
           vse_simul(1, :) = se_bettaf_coef(2:5)';
           vtstat_simul(1, :) = t_value(:)';

           tdist2T = @(t,v) (1-betainc(v/(v+t^2),v/2,0.5));    % 2-tailed t-distribution
           tdist1T = @(t,v) 1-(1-tdist2T(t,v))/2;              % 1-tailed t-distribution
           
           pvalues=zeros(4,1);
           for it=1:4
               pvalues(it)=1-tdist2T(t_value(it),100000);
           end
           vpvalue_simul(1,:) = pvalues(:)'; 

           %compute the conditional mean and standard deviation of log consumption
           mcZ = zeros(1, p.S);
           scZ = zeros(1, p.S);
           
           for j3 = 1 : p.S
              tempindex = mshock_new == j3;
              tempcons = lcons(tempindex);
              mcZ(j3) = mean(tempcons);
              scZ(j3) = std(tempcons);
           end
           mcZ_simul(1, :) = mcZ;
           scZ_simul(1, :) = scZ;
        

           clearvars -except  m_mcZ m_scZ TT nburn  savefilename_simul...
                vkappa nkappa  vbetta_simul mcZ_simul scZ_simul  IR_simul fn_limcom_ni fn_limcom_wi  ...
                vtstat_simul  vse_simul vpvalue_simul

           %-------------------------------------------------------------------
           %----------Simulation for the Information Case----------------------
           %-------------------------------------------------------------------

           for j4 = 1 : nkappa
               load(fn_limcom_wi{j4})
              fprintf('Limited Commitment model with kappa = %f is simulated\n', p.kappa);
              disp('-----------------------------------------------------------');

              %storage matrices
              mshock = zeros(1,TT);       %storage vector for the chain
              mnT = zeros(1, TT);         %storage vector for shock process
              mg = zeros(1, TT);        %storage vector for the promised values
              mh = zeros(1, TT-1);       %storage vector for the delivered consumption today

              %get the shock process
              [~, state] = markov(Q, TT,1,Z(:)');
              state = [[1;zeros(numel(Z)-1,1)] state];

              for j5 = 1 : TT
                 mshock(j5) = find(state(:,j5) == 1); 
              end
              mnT = Z(mshock);


              %initial promised value
              mg(1) = agrid(1);

              for j2 = 1 : TT-1 
                 
                    %interpolate for the consumption
                    tempc = cons(:,mshock(j2));
                    mcons(j2) = interp1(agrid,tempc,mg(j2), 'linear');

                    %interpolate for the asset
                    tempa = asset(:,mshock(j2), mshock(j2+1));
                    mg(j2+1) = interp1(agrid,tempa,mg(j2),'linear');
              end


            %burn the initial simulation
            mcons_new = mcons(nburn+1:end);
            mnT_new = mnT(nburn+1:end-1);
            mshock_new = mshock(nburn+1:end-1);

            %log consumption and income
            lcons = log(mcons_new);
            ly  = log(mnT_new);  


             %compute the unconditional variance of consumption and income
             v_vcvZ = var(lcons)/var(ly);
             IR_simul(j4+ 1) = 1 - var(lcons)/var(ly);

           %Compute regression on future income growth
           lc=log(mcons_new);
           ldc = lc(2:end) - lc(1:end-1);
           ldc=ldc(:);
           ly  = log(mnT_new);
           ldy = ly(2:end) - ly(1:end-1);
           yh = lagmatrix(ly(:),[0, -1, -2 -3 -4]);% help matrix income
           gry = yh(:,2:5) - yh(:,1:4);
           indexgry = isnan(gry(:,end));
           gry(indexgry,:)=[];
           ldcf=ldc(1:length(gry));  
           mxf = [ones(length(ldcf),1), gry];%constant, coeff. on future income growth,  current income growth
           bettaf = (mxf'* mxf)\mxf'* ldcf(:);
           v_bettaf_info_LC = [bettaf(2), bettaf(3) bettaf(4) bettaf(5)]; %coefficient on future income growth
           sef=(ldcf-mxf*bettaf)'*(ldcf-mxf*bettaf);
           betta_sef=(mxf'*mxf).\(sef*ones(5));
           se_bettaf_coef=diag(betta_sef);
           t_value=v_bettaf_info_LC(:)./se_bettaf_coef(2:5);

           vbetta_simul(j4+1, :) = v_bettaf_info_LC;
           vse_simul(j4+1, :) = se_bettaf_coef(2:5)';
           vtstat_simul(j4+1, :) = t_value(:)' ;

           tdist2T = @(t,v) (1-betainc(v/(v+t^2),v/2,0.5));    % 2-tailed t-distribution
           tdist1T = @(t,v) 1-(1-tdist2T(t,v))/2;              % 1-tailed t-distribution
           
           pvalues=zeros(4,1);
           for it=1:4
               pvalues(it)=1-tdist2T(t_value(it),100000);
           end
           vpvalue_simul(j4+1,:) = pvalues(:)';


             %compute the conditional mean and standard deviation
             mcZ = zeros(1,p.S^0.5);
             scZ = zeros(1,p.S^0.5);

             for j7 = 1 : p.S^0.5
                ind = [];
                for j8 = (j7-1)*p.S^0.5 + 1 : j7* p.S^0.5
                    tempindex = find(mshock_new == j8 );
                    ind = [ind;tempindex'];
                end
                tempc = mcons_new(ind);
                mcZ(j7) = mean(log(tempc));
                scZ(j7) = std(log(tempc));
             end

             mcZ_simul(j4+1, :) = mcZ;
             scZ_simul(j4+1, :) = scZ;
            
            Zuni = unique(Z);
            lZuni = log(Zuni);


   
   
           end
    
    disp('Saving results in file named:')
    disp(savefilename_simul)
    
    save(savefilename_simul, 'mcZ_simul', 'scZ_simul', 'Z', 'vkappa', 'Zuni', 'lZuni', 'IR_simul', 'vbetta_simul', 'vse_simul', 'vtstat_simul', 'vpvalue_simul')
   




%Write the results for the limcom model
%--------------------------------------
clearvars -except savefilename_simul


load(savefilename_simul)


vkappa_sim = kron([0;vkappa'], ones(4,1));
variable = repmat((1:4)', size(vbetta_simul,1),1);
complete_results = zeros(length(vkappa_sim), size(vbetta_simul,2));


for i1 = 1 : length(vkappa)+1
   complete_results((i1-1)*4+1, :) = vbetta_simul(i1,:);
   complete_results((i1-1)*4+2, :) = vse_simul(i1,:);
   complete_results((i1-1)*4+3, :) = vtstat_simul(i1,:);
   complete_results((i1-1)*4+4, :) = vpvalue_simul(i1,:);
end


formatspec= '%4.2f & %i & %4.2f & %4.2f & %4.2f & %4.2f\\\\ \n';

complete_results = [vkappa_sim(:), variable(:), complete_results];

   fileID = fopen('limcom_regression_results_capital_R1025_noreturn.txt', 'w');
   fprintf(fileID, formatspec, complete_results');



end