%function simulation()




%0. Preliminaries
%----------------                    
clc; clear all; close all;
addpath('../Results','../../Include/');

%while loading the results the code gives warning that it cannot find
%function handle. This warning is harmless, so I just suppress it. 
warning('off');


%Filenames for the Limcom and Aiyagari
%-------------------------------------
   
    %aiyagari filenames
    vkappa = [0.99];
    nkappa = length(vkappa);
    load('../parameters_pers_aiyagari_ni.mat','p');
    income_states = p.S;
    
    %no information filename
    strnpers = num2str(p.npers);
    strntrans = num2str(p.ntrans);
    strintr = num2str(p.R);
    strbr = num2str(p.kmin);
    fn_aiyagari_ni = strcat('pers_aiyagari_ni');
    fn_aiyagari_ni(fn_aiyagari_ni == '.') = [];
    
    clearvars p
    
    
    
    
    %with information filename
    fn_aiyagari_wi = cell(length(vkappa),1);
    for i1 = 1 : length(vkappa)
       temp = strcat('pers_aiyagari_wi_kappa_',num2str(vkappa(i1)));
       temp(temp == '.') = [];
       fn_aiyagari_wi{i1} = temp;
    end
    
    savefilename_aiyagari = strcat('final_results_aiyagari_capital_simul');
    savefilename_aiyagari(savefilename_aiyagari == '.') = [];
    
    disp('The results will be saved in filename;')
    disp(savefilename_aiyagari);
    



%fix the seed of random number generation
s = RandStream('mcg16807','Seed',10);
RandStream.setGlobalStream(s);


TT =350000;       %length of the simulation
nburn = 100000;    %number of simulation burned
fprintf('Total number of simulation period = %i\n', TT);
fprintf('Number of initial simulations burned = %i\n', nburn);

%storage matrices
mcZ_simul = zeros(nkappa+1, income_states);      %conditional mean of log consumption storage matrix
scZ_simul = zeros(nkappa+1, income_states);      %conditional std. dev. of log consumption storage matrix
maZ_simul = zeros(nkappa+1, income_states);      %conditional mean of asset storage matrix
saZ_simul = zeros(nkappa+1, income_states);      %conditional std. dev. of asset storage matrix
IR_simul = zeros(nkappa+1,1);                    %insurance ratio storage matrix
vbetta_simul = zeros(nkappa+1,4);                %regression coefficient storage matrix
vtstat_simul = zeros(nkappa+1,4);                %t-statistic storage matrix
vse_simul = zeros(nkappa+1,4);                   %standard error storage matrix
vpvalue_simul = zeros(nkappa+1,4);               %pvalue storage matrix

%-----------------------------------------------------------------------
%-------- No Information Economy ---------------------------------------
%-----------------------------------------------------------------------

        
        load pers_aiyagari_ni
            
        disp('-----------------------------------------------------------');
        disp('Aiyagari With Capital No Information model is being simulated');
        disp('-----------------------------------------------------------');
       
        ma = zeros(1, TT);       %storage vector for assets
        mcons = zeros(1, TT);    %storage vector for consumption

        %1.1 Simulate the markov chain
        %-----------------------------
        mnT = [Z(1) markov(Q, TT,1,Z)];

        %1.2 Generate the initial assets
        %-------------------------------
        ma(1) = kgrid(end);

        %1.3 Generate assets and consumption
        %-----------------------------------
        kp = reshape(kp, p.nk, p.S);
        for i3 = 1 : TT-1
           index_y = find(mnT(i3) == Z);
           index_a = find(ma(i3) == kgrid);
           ma(i3+1) = kp(index_a, index_y);
           mcons(i3) = (1 + r)*ma(i3) + mnT(i3) - ma(i3+1);
        end

        %check for the negative consumption
        index = sum(sum(mcons < 0));
        if index > 0
           error('Consumption is negative'); 
        end

        %1.4 Unconditional moments
        %-------------------------    
        mcons_new = mcons(nburn+1:end-1);
        mnT_new = mnT(nburn+1:end-1); 
        ma_new = ma(nburn+1:end);
        vcvZ = var(log(mcons_new(:)))/var(log(mnT_new(:))); %Ratio is too low because variance of consumption is lower than in invariant distribution   
        IR_simul(1) = 1 - var(log(mcons_new(:)))/var(log(mnT_new(:)));
        
        
        
      


% stop
        %1.5 Regression
        %--------------
        lc=log(mcons_new);
        ldc = lc(2:end) - lc(1:end-1);
        ldc=ldc(:);
        ly  = log(mnT_new);
   
   
       %Compute regression on future income growth
       yh = lagmatrix(ly(:),[0, -1, -2 -3 -4]);% help matrix income
       gry = yh(:,2:5) - yh(:,1:4);
       indexgry = isnan(gry(:,end));
       gry(indexgry,:)=[];
       ldcf=ldc(1:length(gry));  
       mxf = [ones(length(ldcf),1), gry];%constant, coeff. on future income growth,  current income growth
       bettaf = (mxf'* mxf)\mxf'* ldcf(:);
       v_bettaf = [bettaf(2), bettaf(3) bettaf(4) bettaf(5)]; %coefficient on future income growth
       sef=(ldcf-mxf*bettaf)'*(ldcf-mxf*bettaf);
       betta_sef=(mxf'*mxf).\(sef*ones(5));
       se_bettaf_coef=diag(betta_sef);
       t_value=v_bettaf(:)./se_bettaf_coef(2:5);
   
       vbetta_simul(1, :) = v_bettaf;
       vse_simul(1, :) = se_bettaf_coef(2:5)';
       vtstat_simul(1, :) = t_value(:)';  

       tdist2T = @(t,v) (1-betainc(v/(v+t^2),v/2,0.5));    % 2-tailed t-distribution
       tdist1T = @(t,v) 1-(1-tdist2T(t,v))/2;              % 1-tailed t-distribution

       pvalues=zeros(4,1);
       for it=1:4
           pvalues(it)=1-tdist2T(t_value(it),100000);
       end
       vpvalue_simul(1,:) = pvalues(:)';

   
   
 

        %1.6 Conditional moments
        %-----------------------
        mcZ = zeros(1, p.S);
        scZ = zeros(1,p.S);
        maZ = zeros(1, p.S);
        saZ = zeros(1,p.S);
        for i1 = 1 : p.S
           index = mnT_new == Z(i1);
           tempc = mcons_new(index);
           tempa = ma_new(index); 
           mcZ(i1) = mean(log(tempc));
           scZ(i1) = std(log(tempc));
           
           maZ(i1) = mean(tempa);
           saZ(i1) = std(tempa);
        end
        mcZ_simul(1,:) = mcZ;
        scZ_simul(1,:) = scZ;
        maZ_simul(1,:) = maZ;
        saZ_simul(1,:) = saZ;


        clearvars -except mcZ_simul scZ_simul maZ_simul saZ_simul vbetta_simul vse_simul...
            vtstat_simul TT nburn vkappa nkappa IR_simul  savefilename_aiyagari...
             fn_aiyagari_ni fn_aiyagari_wi vpvalue_simul

        %2. Aiyagari Endowment with Information Case
        %-------------------------------------------

        for i5 = 1 : nkappa
           cd ..
           cd 'Results'
           load(fn_aiyagari_wi{i5});
           cd ..
           cd 'Statistics_Simulation'
           fprintf('Aiyagari with capital  kappa = %f model is simulated\n',p.kappa);
           disp('-----------------------------------------------------------');

           %Generate the markov chain
           [~,state] = markov(Q, TT,1 , Z(:)');
           state = [[1;zeros(numel(Z)-1,1)] state];
           index = zeros(1,TT);
           for i2 = 1 : TT
               index(i2) = find(state(:,i2) == 1);
           end
           mnT = Z(index);

           %define the storage vector for savings and consumption
           ma = zeros(1, TT);       %storage vector for assets
           mcons = zeros(1, TT);    %storage vector for consumption

           %initialize the savings position
           ma(1) = kgrid(1);
   
           %generate the asset and consumption
           kp = reshape(kp, p.nk, p.S);
           for i3 = 1 : TT-1
              index_a = find(ma(i3) == kgrid);
              ma(i3+1) = kp(index_a, index(i3));
              mcons(i3) = (1 + r)*ma(i3) + mnT(i3) - ma(i3+1);
           end

            %check for the negative consumption
            indexcons = sum(sum(mcons < 0));
            if indexcons > 0
               error('Consumption is negative'); 
            end

            %Unconditional moments
            %--------------------- 
            index_new = index(nburn+1:end-1);
            mcons_new = mcons(nburn+1:end-1);
            mnT_new = mnT(nburn+1:end-1);  
            ma_new = ma(nburn+1:end);
            vcvZ(i5+1) = var(log(mcons_new(:)))/var(log(mnT_new(:)));
            IR_simul(i5+1) = 1 - var(log(mcons_new(:)))/var(log(mnT_new(:)));
         
    
            lc=log(mcons_new);
            ldc = lc(2:end) - lc(1:end-1);
            ldc=ldc(:);
            ly  = log(mnT_new);
   
   
           %Compute regression on future income growth
           yh = lagmatrix(ly(:),[0, -1, -2 -3 -4]);% help matrix income
           gry = yh(:,2:5) - yh(:,1:4);
           indexgry = isnan(gry(:,end));
           gry(indexgry,:)=[];
           ldcf=ldc(1:length(gry));  
           mxf = [ones(length(ldcf),1), gry];%constant, coeff. on future income growth,  current income growth
           bettaf = (mxf'* mxf)\mxf'* ldcf(:);
           v_bettaf = [bettaf(2), bettaf(3) bettaf(4) bettaf(5)]; %coefficient on future income growth
           sef=(ldcf-mxf*bettaf)'*(ldcf-mxf*bettaf);
           betta_sef=(mxf'*mxf).\(sef*ones(5));
           se_bettaf_coef=diag(betta_sef);
           t_value=v_bettaf(:)./se_bettaf_coef(2:5);
   
    
           vbetta_simul(i5+1, :) = v_bettaf;
           vse_simul(i5+1, :) = se_bettaf_coef(2:5)';
           vtstat_simul(i5+1, :) = t_value(:)';
    
           tdist2T = @(t,v) (1-betainc(v/(v+t^2),v/2,0.5));    % 2-tailed t-distribution
           tdist1T = @(t,v) 1-(1-tdist2T(t,v))/2;              % 1-tailed t-distribution

           pvalues=zeros(4,1);
           for it=1:4
               pvalues(it)=1-tdist2T(t_value(it),100000);
           end
           vpvalue_simul(i5+1,:) = pvalues(:)';
    


            %conditional moments
            %-------------------
            mcZ = zeros(1,p.S^0.5);
            scZ = zeros(1,p.S^0.5);
            maZ= mcZ;
            saZ = scZ;
            
            for i6 = 1 : p.S^0.5
               ind = [];
               for i7 = (i6-1)*p.S^0.5 + 1 : i6* p.S^0.5
                   tempindex = find(index_new == i7 );
                   ind = [ind;tempindex'];
               end
               tempc = mcons_new(ind);
               mcZ(i6) = mean(log(tempc));
               scZ(i6) = std(log(tempc));
               
               tempa = ma_new(ind);
               maZ(i6) = mean(tempa);
               saZ(i6) = std(tempa);
               
            end
            mcZ_simul(i5+1,:) = mcZ;
            scZ_simul(i5+1,:) = scZ;
            
            maZ_simul(i5+1,:) = maZ;
            saZ_simul(i5+1,:) = saZ;
            
        end

        Zuni = unique(Z);
        lZuni = log(Zuni);

disp('Saving results in file named:')
disp(savefilename_aiyagari)
cd '../Results'
save(savefilename_aiyagari, 'mcZ_simul', 'scZ_simul', 'maZ_simul', 'saZ_simul', 'Z', ...
    'vkappa', 'Zuni', 'lZuni','IR_simul', 'vbetta_simul', 'vse_simul', 'vtstat_simul', 'vpvalue_simul');
cd ..
cd 'Statistics_Simulation'



    
%Write the results for the aiyagari model
%----------------------------------------
cd '../Results'
load(savefilename_aiyagari)
cd ..
cd 'Statistics_Simulation'
vkappa_sim = kron([0;vkappa'], ones(4,1));
variable = repmat((1:4)', size(vbetta_simul,1),1);
complete_results = zeros(length(vkappa_sim), size(vbetta_simul,2));


for i1 = 1 : length(vkappa)+1
   complete_results((i1-1)*4+1, :) = vbetta_simul(i1,:);
   complete_results((i1-1)*4+2, :) = vse_simul(i1,:);
   complete_results((i1-1)*4+3, :) = vtstat_simul(i1,:);
   complete_results((i1-1)*4+4, :) = vpvalue_simul(i1,:);
end


formatspec= '%4.2f & %i & %4.2f & %4.2f & %4.2f & %4.2f\\\\ \n';

complete_results = [vkappa_sim(:), variable(:), complete_results];
cd '../Results'
   fileID = fopen('aiyagari_with_capital_regression_results.txt', 'w');
   fprintf(fileID, formatspec, complete_results');
cd .. 

cd 'Statistics_Simulation'



%end