function parameters_aiyagari(chosen_model, kappa)

%parameters_aiyagari(chosen_model, kappa) sets the parameters for the
%Aiyagari model with capital.
%
%Inputs: 
%       chosen_model        Choice between  information/no information
%       kappa               The information parameter
%
%Output:
%       None
%
%Author: Swapnil Singh and Christian Stoltenberg
%
%Version:
%   February 3, 2019
%



%0. Declare parameters
%---------------------

        %0.1 Parameters common to all models
        %-----------------------------------
        p.R = 1.025;                        %Interest rate
        p.sigma=1;                          %Parameter specification of utility function
        p.alpha = 0.3;                      %capital share 
        p.delta=0.1357;                     %Depreciation rate
        p.AA = 1.0644;                      %production technology parameter
        p.nk = 300;                         %Number of grid points for capital
        p.mid = 200;                       %Number of grid points until mid-point
        p.kmid = 5;                           %Midpoint opf capital grid
        p.kmin = 2.12;                        %Lowest value on the capital grid
        p.kmax = 20;                        %Highest value on the capital grid
        p.pcrit=1e-10;                      %Invariant distribution, minimum probability of state to be kept
        p.crit = 1e-8;                      %convergence criteria for value function in solution
        p.npers = 3;                        %Number of persistent states
        p.ntrans = 2;                       %Number of transitory states
        p.S = p.npers * p.ntrans;           %Total number of income states
        p.rho = 0.9989;                     %Persistence parameter
        p.tauchen = 0;
        if nargin == 2 && strcmp(chosen_model, 'pers_aiyagari_wi')
           p.kappa = kappa;
           p.Sinf = p.S^2;
        end
        
        %capital grid
        kgrid = [linspace(p.kmin,p.kmid,p.mid) linspace(p.kmid+1e-3, p.kmax, p.nk-p.mid)]';
        
%1. Save the parameters
%----------------------
cd 'Aiyagari'
    filename = strcat('parameters_', chosen_model);
    save(filename);
cd ..


end